# postcreate script. You could setup a workflow here for example
from cubicweb import _

wf = add_workflow('invoice workflow', 'Invoice')
waiting_command = wf.add_state(_('waiting command'), initial=True)
planned = wf.add_state(_(u'planned'))
toissue = wf.add_state(_(u'to issue'))
tosend = wf.add_state(_(u'to send'))
sent = wf.add_state(_(u'sent'))
payed = wf.add_state(_(u'payed'))

wf.add_transition(_('command received'), waiting_command, planned)
wf.add_transition(_(u'work done'), planned, toissue)
wf.add_transition(_(u'issue'), toissue, tosend)
wf.add_transition(_(u'send'), tosend, sent)
wf.add_transition(_(u'pay'), sent, payed)



"""this contains the template-specific entities' classes"""

from cubicweb.entities import AnyEntity, fetch_config


class Invoice(AnyEntity):
    __regid__ = 'Invoice'
    fetch_attrs, cw_fetch_order = fetch_config(['num'])

    def dc_title(self):
        return u"%s #%s" % (self.type, self.num.upper())

    def dc_long_title(self):
        _ = self._cw._
        return u"%s (%s+%s) %s %s, %s %s, %s %s - %s" % (
            self.dc_title(), self.amount, self.taxes,
            _('emited on'), self.printable_value('emit_date'),
            _('due on'), self.printable_value('due_date'),
            _('paid on'), self.printable_value('pay_date'),
            self.cw_adapt_to('IWorkflowable').state)


class Account(AnyEntity):
    __regid__ = 'Account'
    fetch_attrs, cw_fetch_order = fetch_config(['label', 'account'])

    def dc_title(self):
        return u'%s - %s' % (self.account, self.label)



# Invoice workflow migration

waiting_command = add_state(_('waiting command'), 'Invoice', initial=True)
planned = rql('State S WHERE S state_of ET, ET name "Invoice", S name "planned"')[0][0]
add_transition(_('command received'), 'Invoice', (waiting_command,), planned)



from yams.buildobjs import EntityType, SubjectRelation, String, Date, Float
from cubicweb.schema import WorkflowableEntityType
from cubicweb import _


class Invoice(WorkflowableEntityType):
    num = String(required=True, fulltextindexed=True, indexed=True,
                 maxsize=16, description=_('invoice number'))
    emit_date = Date(description=_('emission date'))
    due_date = Date(description=_('due date'))
    pay_date = Date(description=_('payment date'))
    type = String(required=True, internationalizable=True,
                  vocabulary=(_('invoice'), _('expenses'), _('credit')),
                  description=_("invoice type"), default='invoice')
    amount = Float(description=_('total amount without taxes'), required=True)
    taxes = Float(description=_('taxes on total amount'), required=True)

    credit_account = SubjectRelation('Account', cardinality='?*',
                                     description=_('account to credit'))
    debit_account = SubjectRelation('Account', cardinality='?*',
                                    description=_('account to debit'))


class Account(EntityType):
    __permissions__ = {'read': ('users', 'managers'),
                       'add': ('managers', ),
                       'update': ('managers',),
                       'delete': ('managers',),
                       }
    label = String(required=True, maxsize=128)
    account = String(required=True, maxsize=16)



# pylint: disable=W0622
"""cubicweb-invoice application packaging information"""


modname = 'cubicweb_invoice'
distname = 'cubicweb-invoice'

numversion = (1, 0, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'Logilab'
author_email = 'contact@logilab.fr'
description = 'invoice component for the CubicWeb framework'
web = 'http://www.cubicweb.org/project/%s' % distname

__depends__ = {
    'cubicweb': '>=4.0.0,<5.0.0',
    'cubicweb_web': '>=1.0.0,<2.0.0',
}
__recommends__ = {}

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python :: 3',
    'Programming Language :: JavaScript',
]



"""template-specific forms/views/actions/components"""
from cubicweb.predicates import is_instance
from cubicweb_web.facet import RangeFacet, DateRangeFacet
from cubicweb_web.views import uicfg

uicfg.autoform_section.tag_subject_of(('Invoice', 'credit_account', '*'), 'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('Invoice', 'debit_account', '*'), 'muledit', 'attributes')
uicfg.autoform_section.tag_subject_of(('Invoice', 'credit_account', '*'), 'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('Invoice', 'debit_account', '*'), 'muledit', 'attributes')


class InvoiceEmitDateFacet(DateRangeFacet):
    __regid__ = 'invoice.emit_date.facet'
    __select__ = DateRangeFacet.__select__ & is_instance('Invoice')
    rtype = 'emit_date'


class InvoiceDueDateFacet(DateRangeFacet):
    __regid__ = 'invoice.due_date.facet'
    __select__ = DateRangeFacet.__select__ & is_instance('Invoice')
    rtype = 'due_date'


class InvoicePayDateFacet(DateRangeFacet):
    __regid__ = 'invoice.pay_date.facet'
    __select__ = DateRangeFacet.__select__ & is_instance('Invoice')
    rtype = 'pay_date'


class InvoiceAmountFacet(RangeFacet):
    __regid__ = 'invoice.amount.facet'
    __select__ = RangeFacet.__select__ & is_instance('Invoice')
    rtype = 'amount'
